@@ -22,14 +22,12 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE. */
 
-#include "options.h"
-#include "libtomcrypt/mycrypt.h"
-#include "util.h"
-#include "buffer.h"
-
 #include "algo.h"
 
-/* Mapping of ssh ciphers to libtomcrypt ciphers, with blocksizes etc.
+/* This file (algo.c) organises the ciphers which can be used, and is used to
+ * decide which ciphers/hashes/compression/signing to use during key exchange*/
+
+/* Mappings for ciphers, parameters are
    {&cipher_desc, keysize, blocksize} */
 
 #ifdef DROPBEAR_AES128_CBC
@@ -49,8 +47,9 @@ const struct dropbear_cipher dropbear_3des =
 	{&des3_desc, 24, 8};
 #endif
 
+/* used to indicate no encryption, as defined in rfc2410 */
 const struct dropbear_cipher dropbear_nocipher =
-	{NULL, 16, 8}; /* used initially */
+	{NULL, 16, 8}; 
 
 /* Mapping of ssh hashes to libtomcrypt hashes, including keysize etc.
    {&hash_desc, keysize, hashsize} */
@@ -83,7 +82,7 @@ algo_type sshciphers[] = {
 #ifdef DROPBEAR_3DES_CBC
 	{"3des-cbc", 0, (void*)&dropbear_3des, 1},
 #endif
-	{0}
+	{NULL, 0, NULL, 0}
 };
 
 algo_type sshhashes[] = {
@@ -93,7 +92,7 @@ algo_type sshhashes[] = {
 #ifdef DROPBEAR_MD5_HMAC
 	{"hmac-md5", 0, (void*)&dropbear_md5, 1},
 #endif
-	{0}
+	{NULL, 0, NULL, 0}
 };
 
 algo_type sshcompress[] = {
@@ -101,7 +100,7 @@ algo_type sshcompress[] = {
 #ifndef DISABLE_ZLIB
 	{"zlib", DROPBEAR_COMP_ZLIB, NULL, 1},
 #endif
-	{0}
+	{NULL, 0, NULL, 0}
 };
 
 algo_type sshhostkey[] = {
@@ -111,12 +110,12 @@ algo_type sshhostkey[] = {
 #ifdef DROPBEAR_DSS
 	{"ssh-dss", DROPBEAR_SIGNKEY_DSS, NULL, 1},
 #endif
-	{0}
+	{NULL, 0, NULL, 0}
 };
 
 algo_type sshkex[] = {
 	{"diffie-hellman-group1-sha1", DROPBEAR_KEX_DH_GROUP1, NULL, 1},
-	{0}
+	{NULL, 0, NULL, 0}
 };
 
 
@@ -137,7 +136,7 @@ void crypto_init() {
 #ifdef DROPBEAR_3DES_CBC
 		&des3_desc,
 #endif
-	NULL
+		NULL
 	};
 
 	const struct _hash_descriptor *reghashes[] = {
@@ -161,12 +160,11 @@ void crypto_init() {
 			dropbear_exit("error registering crypto");
 		}
 	}
-
 }
 
 /* returns DROPBEAR_SUCCESS if we have a match for algo, DROPBEAR_FAILURE
  * otherwise */
-int have_algo(char* algo, int algolen, algo_type algos[]) {
+int have_algo(char* algo, size_t algolen, algo_type algos[]) {
 
 	int i = 0;
 	while (algos[i].name != NULL) {
@@ -181,36 +179,37 @@ int have_algo(char* algo, int algolen, algo_type algos[]) {
 }
 
 
-
-/* hashlist is a comma seperated, null terminated list of hashes.
- * The first item in the list which also has a local entry will be chosen */
+/* match the first algorithm in the comma-seperated list in buf which is
+ * also in localalgos[], or return NULL on failure. */
 algo_type * buf_match_algo(buffer* buf, algo_type localalgos[]) {
 
 	unsigned char * algolist = NULL;
-	unsigned char* alterlist = NULL;
 	unsigned char * remotealgos[MAX_PROPOSED_ALGO];
-	unsigned int pos1, pos2;
 	unsigned int len;
 	unsigned int count, i, j;
-	algo_type * ret;
+	algo_type * ret = NULL;
 
 	/* get the comma-seperated list from the buffer ie "algo1,algo2,algo3" */
 	algolist = buf_getstring(buf, &len);
 	if (len > MAX_PROPOSED_ALGO*(MAX_NAME_LEN+1)) {
-		ret = NULL;
 		goto out; /* just a sanity check, no other use */
 	}
-	alterlist = strdup((char*)algolist);
 
 	/* remotealgos will contain a list of the strings parsed out */
-	count = 0;
-	pos1 = 0;
-	for (pos2 = 0; pos2 <= len; pos2++) {
-		if (alterlist[pos2] == ',' || alterlist[pos2] == '\0') {
-			remotealgos[count] = &alterlist[pos1];
-			alterlist[pos2] = '\0';
+	/* We will have at least one string (even if it's just "") */
+	remotealgos[0] = algolist;
+	count = 1;
+	/* Iterate through, replacing ','s with NULs, to split it into
+	 * words. */
+	for (i = 0; i < len; i++) {
+		if (algolist[i] == '\0') {
+			/* someone is trying something strange */
+			goto out;
+		}
+		if (algolist[i] == ',' && i != len) {
+			algolist[i] = '\0';
+			remotealgos[count] = &algolist[i+1];
 			count++;
-			pos1 = pos2+1; /* this is safe since we check pos2 next loop */
 		}
 		if (count == MAX_PROPOSED_ALGO) {
 			break;
@@ -231,15 +230,13 @@ algo_type * buf_match_algo(buffer* buf, algo_type localalgos[]) {
 			}
 		}
 	}
-	ret = NULL;
 
 out:
-	m_free(alterlist);
 	m_free(algolist);
 	return ret;
 }
 
-/* output a comma seperated list of algorithms to a buffer */
+/* Output a comma seperated list of algorithms to a buffer */
 void buf_put_algolist(buffer * buf, algo_type localalgos[]) {
 
 	unsigned int pos = 0, i, len;
