@@ -25,13 +25,10 @@
 #ifndef _DEBUG_H_
 #define _DEBUG_H_
 
-#include <stdio.h>
-#include <assert.h>
+#include "includes.h"
 
 /* Debugging */
 
-/*#define DISABLE_X11FWD*/
-
 /*#define DEBUG_KEXHASH*/
 /*#define DEBUG_RSA*/
 
