@@ -39,7 +39,9 @@
 /*#define DISABLE_X11FWD*/
 
 /* Disable TCP Fowarding */
-/*#define DISABLE_TCPFWD*/
+/* NOTE: TCP forwarding is still work-in-progress, -L forwarding should work,
+ * enable TCP forwarding at your own risk. */
+#define DISABLE_TCPFWD
 
 /* Disable Authentication Agent Forwarding */
 /*#define DISABLE_AGENTFWD*/
@@ -112,7 +114,7 @@
  * You shouldn't edit below here unless you know you need to.
  *******************************************************************/
 
-#define DROPBEAR_VERSION "0.33"
+#define DROPBEAR_VERSION "0.36"
 #define LOCAL_IDENT "SSH-2.0-dropbear_" DROPBEAR_VERSION
 #define PROGNAME "dropbear"
 
@@ -151,6 +153,9 @@
 /* SHA512 is 64 bytes == 512 bits */
 #define SHA512_HASH_SIZE 64
 
+/* largest of MD5 and SHA1 */
+#define MAX_MAC_LEN SHA1_HASH_SIZE
+
 
 #define MAX_KEY_LEN 24 /* 3DES requires a 24 byte key */
 #define MAX_IV_LEN 20 /* must be same as max blocksize, 
@@ -178,6 +183,8 @@
 
 #define MAX_BANNER_SIZE 2000 /* this is 25*80 chars, any more is foolish */
 
+#define MAX_STRING_LEN 1400 /* ~ MAX_PROPOSED_ALGO * MAX_NAME_LEN */
+
 #define DEV_URANDOM "/dev/urandom"
 
 /* the number of NAME=VALUE pairs to malloc for environ, if we don't have
@@ -187,6 +194,16 @@
 #define MAX_CMD_LEN 1024 /* max length of a command */
 #define MAX_TERM_LEN 200 /* max length of TERM name */
 
+#define MAX_HOST_LEN 254 /* max hostname len for tcp fwding */
+
+#define DROPBEAR_MAX_PORTS 10 /* max number of ports which can be specified,
+								 ipv4 and ipv6 don't count twice */
+
 #define _PATH_TTY "/dev/tty"
 
+#ifdef DISABLE_TCPFWD
+#define DISABLE_LOCALTCPFWD
+#define DISABLE_REMOTETCPFWD
+#endif
+
 #endif /* _OPTIONS_H_ */
